import requests

from dk_analyzer.models import Event


def get_access_token(client_id: str, client_secret: str) -> str:
    response = requests.post(
        "https://www.warcraftlogs.com/oauth/token",
        data={"grant_type": "client_credentials"},
        auth=(client_id, client_secret),
        timeout=5,
    )
    return response.json()["access_token"]


def fetch_report(report_id: str, fight_id: int, access_token: str) -> list[Event]:
    headers = {
        "Content-Type": "application/json",
        "Authorization": f"Bearer {access_token}",
    }
    body = f"""
query {{
    reportData {{
        report(code:"{report_id}"){{
            events(
                fightIDs:{fight_id},
                abilityID:45470,
                sourceClass:"DEATHKNIGHT",
                dataType:Healing,
                includeResources:true,
                limit:9000
            ) {{
                data
                nextPageTimestamp
            }}
        }}
    }}
}}
    """
    response = requests.post(
        "https://www.warcraftlogs.com/api/v2/client",
        headers=headers,
        json={"query": body},
        params={"code": report_id},
        timeout=5,
    )
    json_report = response.json()["data"]["reportData"]["report"]
    json_events = json_report["events"]["data"]
    return [Event(json_event) for json_event in json_events]



import logging
from typing import Any


class Event:
    def __init__(self, event: dict[str, Any]) -> None:
        self._event = event
        self.current_hp = int(event["hitPoints"])
        self.max_hp = int(event["maxHitPoints"])
        self.heal_amount = int(event["amount"])
        self.hp_before = self.current_hp - self.heal_amount

    def hp_percent(self) -> float:
        if self.hp_before - self.max_hp > 0:
            logging.error("Higher than 100%% HP?? %s", self._event)  # noqa: WPS323
            return 100.0
        return (self.hp_before / self.max_hp) * 100

    def rp(self) -> float:
        return self._event["classResources"][0]["amount"] / 10

    def is_cast_by_player(self) -> bool:
        return self._event["classResources"][0]["max"] != 0



import statistics

from matplotlib import pyplot

from dk_analyzer.models import Event


class Graph:
    def __init__(self, target_hp_percent: float, target_rp_percent: float) -> None:
        self.target_hp_percent = target_hp_percent
        self.target_rp_percent = target_rp_percent
        self._min_ds_cost = 35
        self._max_hp_percent = 100

    def plot(self, events: list[Event]) -> None:
        hp_list: list[float] = []
        rp_list: list[float] = []
        for event in events:
            if not event.is_cast_by_player():
                continue
            hp_list.append(event.hp_percent())
            rp_list.append(event.rp())

        pyplot.grid(zorder=0)
        pyplot.scatter(
            x=rp_list,
            y=hp_list,
            marker="o",
            zorder=4,
        )
        pyplot.xlabel("RP")
        pyplot.ylabel("HP%")
        self._add_visual_aid()
        self._add_title(hp_list, rp_list)
        pyplot.show()

    def _add_visual_aid(self, thickness: float = 3) -> None:
        pyplot.vlines(
            self.target_rp_percent,
            ymin=0,
            ymax=self._max_hp_percent,
            colors="#FE2A17",
            linewidth=thickness,
            zorder=3,
            label=f"{self.target_rp_percent} RP",
        )
        pyplot.hlines(
            self.target_hp_percent,
            xmin=self._min_ds_cost,
            xmax=self.target_rp_percent,
            colors="#FEBD0D",
            linewidth=thickness,
            zorder=2,
            label=f"{self.target_hp_percent}% HP",
        )
        pyplot.fill_between(
            [self._min_ds_cost, self.target_rp_percent],
            self.target_hp_percent,
            self._max_hp_percent,
            color="black",
            alpha=0.1,
        )

    def _add_title(self, hp_list: list[float], rp_list: list[float]) -> None:
        mean_rp = round(statistics.mean(rp_list))
        mean_hp = round(statistics.mean(hp_list))
        title = f"DS Usage (mean RP: {mean_rp}, mean HP: {mean_hp}%)"
        pyplot.title(title)



import typer

from dk_analyzer.graph_plotter import Graph
from dk_analyzer.warcraftlogs import fetch_report
from dk_analyzer.warcraftlogs import get_access_token

app = typer.Typer()


@app.command()
def analyze_ds_usage(
    report_id: str = typer.Option("jbapg7M32FTn16Rd"),
    fight_id: int = typer.Option(1),
    rp_dump_target: float = typer.Option(95),
    ds_hp_percent: float = typer.Option(70),
    client_id: str = typer.Argument(..., envvar="CLIENT_ID"),
    client_secret: str = typer.Argument(..., envvar="CLIENT_SECRET"),
) -> None:
    events = fetch_report(
        report_id=report_id,
        fight_id=fight_id,
        access_token=get_access_token(client_id, client_secret),
    )
    Graph(target_hp_percent=ds_hp_percent, target_rp_percent=rp_dump_target).plot(events)


if __name__ == "__main__":
    app()
